/* Any copyright is dedicated to the Public Domain.
 * http://creativecommons.org/publicdomain/zero/1.0/ */

/* eslint-disable */

window.addEventListener("load", function () {
  document.getElementById("");
});
